@@ -10,15 +10,14 @@
 #include <string.h>
 #include <signal.h>
 #include <unistd.h>
-#include <libgnome/libgnome.h>
 #include <libbonobo.h>
 #include <gdk/gdkx.h>
 
 #include "button-widget.h"
 #include "drawer.h"
 #include "launcher.h"
-#include "menu-util.h"
 #include "menu.h"
+#include "panel-addto.h"
 #include "panel-gconf.h"
 #include "panel-config-global.h"
 #include "panel-applet-frame.h"
@@ -35,9 +34,6 @@
 
 #define SMALL_ICON_SIZE 20
 
-static GtkWidget *panel_applet_create_menu (AppletInfo  *info,
-					    PanelWidget *panel_widget);
-
 static GSList *registered_applets = NULL;
 static GSList *queued_position_saves = NULL;
 static guint   queued_position_source = 0;
@@ -158,8 +154,7 @@ panel_applet_recreate_menu (AppletInfo	*info)
 	}
 
 	gtk_widget_unref (info->menu);
-	info->menu = panel_applet_create_menu (info,
-					       PANEL_WIDGET (info->widget->parent));
+	info->menu = panel_applet_create_menu (info);
 }
 
 static void
@@ -195,12 +190,18 @@ applet_callback_callback (GtkWidget      *widget,
 
 	switch (menu->info->type) {
 	case PANEL_OBJECT_LAUNCHER:
-		if (!strcmp (menu->name, "properties"))
-			launcher_properties (
-				menu->info->data, screen);
+		if (!strcmp (menu->name, "launch"))
+			launcher_launch (menu->info->data, widget);
+		else if (!strcmp (menu->name, "properties"))
+			launcher_properties (menu->info->data, screen);
 		break;
 	case PANEL_OBJECT_DRAWER: 
-		if (strcmp (menu->name, "properties")==0) {
+		if (strcmp (menu->name, "add") == 0) {
+			Drawer *drawer = menu->info->data;
+
+			panel_addto_present (GTK_MENU_ITEM (widget),
+					     panel_toplevel_get_panel_widget (drawer->toplevel));
+		} else if (strcmp (menu->name, "properties") == 0) {
 			Drawer *drawer = menu->info->data;
 
 			panel_properties_dialog_present (drawer->toplevel);
@@ -235,7 +236,20 @@ applet_callback_callback (GtkWidget      *widget,
 }
 
 static void
-applet_menu_deactivate (GtkWidget *w, AppletInfo *info)
+applet_menu_show (GtkWidget *w,
+		  AppletInfo *info)
+{
+	PanelWidget *panel_widget;
+
+	panel_widget = PANEL_WIDGET (info->widget->parent);
+
+	panel_toplevel_push_autohide_disabler (panel_widget->toplevel);
+}
+
+
+static void
+applet_menu_deactivate (GtkWidget *w,
+			AppletInfo *info)
 {
 	PanelWidget *panel_widget;
 
@@ -399,21 +413,25 @@ add_to_submenus (AppletInfo *info,
 	g_free(n);
 }
 
-static GtkWidget *
-panel_applet_create_menu (AppletInfo  *info,
-			  PanelWidget *panel_widget)
+GtkWidget *
+panel_applet_create_menu (AppletInfo *info)
 {
-	GtkWidget *menu;
-	GtkWidget *menuitem;
-	GList     *l;
-	gboolean   added_anything = FALSE;
+	GtkWidget   *menu;
+	GtkWidget   *menuitem;
+	GList       *l;
+	PanelWidget *panel_widget;
+	gboolean     added_anything = FALSE;
+
+	panel_widget = PANEL_WIDGET (info->widget->parent);
 
 	menu = g_object_ref (panel_create_menu ());
 	gtk_object_sink (GTK_OBJECT (menu));
 
-	/* connect the deactivate signal, so that we can "re-allow" 
-	 * autohide when the menu is deactivated.
+	/* connect the show & deactivate signal, so that we can "disallow" and
+	 * "re-allow" autohide when the menu is shown/deactivated.
 	 */
+	g_signal_connect (menu, "show",
+			  G_CALLBACK (applet_menu_show), info);
 	g_signal_connect (menu, "deactivate",
 			  G_CALLBACK (applet_menu_deactivate), info);
 
@@ -438,7 +456,7 @@ panel_applet_create_menu (AppletInfo  *info,
 
 		lockable = panel_applet_lockable (info);
 		movable = panel_applet_can_freely_move (info);
-		removable = panel_profile_list_is_writable (PANEL_GCONF_OBJECTS);
+		removable = panel_profile_id_lists_are_writable ();
 
 		locked = panel_widget_get_applet_locked (panel_widget, info->widget);
 
@@ -513,6 +531,48 @@ panel_applet_menu_set_recurse (GtkMenu     *menu,
 	g_list_free (children);
 }
 
+void
+panel_applet_position_menu (GtkMenu   *menu,
+			    int       *x,
+			    int       *y,
+			    gboolean  *push_in,
+			    GtkWidget *applet)
+{
+	GtkRequisition  requisition;
+	GdkScreen      *screen;
+	int             menu_x = 0;
+	int             menu_y = 0;
+
+	g_return_if_fail (PANEL_IS_WIDGET (applet->parent));
+
+	screen = gtk_widget_get_screen (applet);
+
+	gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
+
+	gdk_window_get_origin (applet->window, &menu_x, &menu_y);
+
+	if (GTK_WIDGET_NO_WINDOW (applet)) {
+		menu_x += applet->allocation.x;
+		menu_y += applet->allocation.y;
+	}
+
+	if (PANEL_WIDGET (applet->parent)->orient == GTK_ORIENTATION_HORIZONTAL) {
+		if (menu_y > gdk_screen_get_height (screen) / 2)
+			menu_y -= requisition.height;
+		else
+			menu_y += applet->allocation.height;
+	} else {
+		if (menu_x > gdk_screen_get_width (screen) / 2)
+			menu_x -= requisition.width;
+		else
+			menu_x += applet->allocation.width;
+	}
+
+	*x = menu_x;
+	*y = menu_y;
+	*push_in = TRUE;
+}
+
 static void
 applet_show_menu (AppletInfo     *info,
 		  GdkEventButton *event)
@@ -524,13 +584,11 @@ applet_show_menu (AppletInfo     *info,
 	panel_widget = PANEL_WIDGET (info->widget->parent);
 
 	if (info->menu == NULL)
-		info->menu = panel_applet_create_menu (info, panel_widget);
+		info->menu = panel_applet_create_menu (info);
 
 	if (info->menu == NULL)
 		return;
 
-	panel_toplevel_push_autohide_disabler (panel_widget->toplevel);
-
 	panel_applet_menu_set_recurse (GTK_MENU (info->menu),
 				       "menu_panel",
 				       info->widget->parent);
@@ -544,7 +602,7 @@ applet_show_menu (AppletInfo     *info,
 	gtk_menu_popup (GTK_MENU (info->menu),
 			NULL,
 			NULL,
-			(GtkMenuPositionFunc) panel_position_applet_menu,
+			(GtkMenuPositionFunc) panel_applet_position_menu,
 			info->widget,
 			event->button,
 			event->time);
@@ -887,7 +945,6 @@ panel_applet_save_position (AppletInfo *applet_info,
 	PanelGConfKeyType  key_type;
 	GConfClient       *client;
 	PanelWidget       *panel_widget;
-	const char        *profile;
 	const char        *key;
 	const char        *toplevel_id;
 	char              *old_toplevel_id;
@@ -915,7 +972,6 @@ panel_applet_save_position (AppletInfo *applet_info,
 		return;
 
 	client  = panel_gconf_get_client ();
-	profile = panel_profile_get_name ();
 
 	key_type = applet_info->type == PANEL_OBJECT_BONOBO ? PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS;
 	
@@ -924,7 +980,7 @@ panel_applet_save_position (AppletInfo *applet_info,
 	/* FIXME: Instead of getting keys, comparing and setting, there
 	   should be a dirty flag */
 
-	key = panel_gconf_full_key (key_type, profile, id, "toplevel_id");
+	key = panel_gconf_full_key (key_type, id, "toplevel_id");
 	old_toplevel_id = gconf_client_get_string (client, key, NULL);
 	if (old_toplevel_id == NULL || strcmp (old_toplevel_id, toplevel_id) != 0)
 		gconf_client_set_string (client, key, toplevel_id, NULL);
@@ -937,7 +993,7 @@ panel_applet_save_position (AppletInfo *applet_info,
 
 	right_stick = panel_is_applet_right_stick (applet_info->widget) ? 1 : 0;
 	key = panel_gconf_full_key (
-			key_type, profile, id, "panel_right_stick");
+			key_type, id, "panel_right_stick");
 	if (gconf_client_key_is_writable (client, key, NULL) &&
 	    (gconf_client_get_bool (client, key, NULL) ? 1 : 0) != right_stick)
 		gconf_client_set_bool (client, key, right_stick, NULL);
@@ -946,13 +1002,13 @@ panel_applet_save_position (AppletInfo *applet_info,
 	if (right_stick && !panel_widget->packed)
 		position = panel_widget->size - position;
 
-	key = panel_gconf_full_key (key_type, profile, id, "position");
+	key = panel_gconf_full_key (key_type, id, "position");
 	if (gconf_client_key_is_writable (client, key, NULL) &&
 	    gconf_client_get_int (client, key, NULL) != position)
 		gconf_client_set_int (client, key, position, NULL);
 	
 	locked = panel_widget_get_applet_locked (panel_widget, applet_info->widget) ? 1 : 0;
-	key = panel_gconf_full_key (key_type, profile, id, "locked");
+	key = panel_gconf_full_key (key_type, id, "locked");
 	if (gconf_client_get_bool (client, key, NULL) ? 1 : 0 != locked)
 		gconf_client_set_bool (client, key, locked, NULL);
 }
@@ -1142,7 +1198,6 @@ panel_applet_can_freely_move (AppletInfo *applet)
 	PanelWidget       *panel_widget;
 	GConfClient       *client;
 	PanelGConfKeyType  key_type;
-	const char        *profile;
 	const char        *key;
 
 	if (panel_lockdown_get_locked_down ())
@@ -1151,19 +1206,18 @@ panel_applet_can_freely_move (AppletInfo *applet)
 	panel_widget = PANEL_WIDGET (applet->widget->parent);
 
 	client  = panel_gconf_get_client ();
-	profile = panel_profile_get_name ();
 	
 	key_type = (applet->type == PANEL_OBJECT_BONOBO) ? PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS;
        
-	key = panel_gconf_full_key (key_type, profile, applet->id, "position");
+	key = panel_gconf_full_key (key_type, applet->id, "position");
 	if (!gconf_client_key_is_writable (client, key, NULL))
 		return FALSE;
 
-	key = panel_gconf_full_key (key_type, profile, applet->id, "toplevel_id");
+	key = panel_gconf_full_key (key_type, applet->id, "toplevel_id");
 	if (!gconf_client_key_is_writable (client, key, NULL))
 		return FALSE;
 
-	key = panel_gconf_full_key (key_type, profile, applet->id, "panel_right_stick");
+	key = panel_gconf_full_key (key_type, applet->id, "panel_right_stick");
 	if (!gconf_client_key_is_writable (client, key, NULL))
 		return FALSE;
 
@@ -1176,7 +1230,6 @@ panel_applet_lockable (AppletInfo *applet)
 	GConfClient        *client;
 	PanelWidget        *panel_widget;
 	PanelGConfKeyType   key_type;
-	const char         *profile;
 	const char         *key;
 
 	if (panel_lockdown_get_locked_down ())
@@ -1185,11 +1238,10 @@ panel_applet_lockable (AppletInfo *applet)
 	panel_widget = PANEL_WIDGET (applet->widget->parent);
 
 	client  = panel_gconf_get_client ();
-	profile = panel_profile_get_name ();
 	
 	key_type = (applet->type == PANEL_OBJECT_BONOBO) ? PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS;
 
-	key = panel_gconf_full_key (key_type, profile, applet->id, "locked");
+	key = panel_gconf_full_key (key_type, applet->id, "locked");
 
 	return gconf_client_key_is_writable (client, key, NULL);
 }
