@@ -30,20 +30,18 @@
 #include <glib/gi18n.h>
 
 #include "panel-util.h"
+#include "panel-background.h"
 #include "panel-action-button.h"
 #include "panel-stock-icons.h"
 #include "panel-recent.h"
 #include "applet.h"
 #include "menu.h"
-#include "menu-util.h"
 #include "panel-globals.h"
 #include "panel-profile.h"
 #include "panel-lockdown.h"
 
 #define PANEL_MENU_BAR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_MENU_BAR, PanelMenuBarPrivate))
 
-#define MENU_FLAGS (MAIN_MENU_SYSTEM)
-
 struct _PanelMenuBarPrivate {
 	AppletInfo            *info;
 	PanelWidget           *panel;
@@ -54,49 +52,35 @@ struct _PanelMenuBarPrivate {
 };
 
 static GObjectClass *parent_class;
-static void panel_menu_bar_append_actions_menu (PanelMenuBar *menubar);
+static GtkWidget *panel_menu_bar_create_actions_menu (PanelMenuBar *menubar);
 
 static void
-panel_menu_bar_show_applications_menu (PanelMenuBar *menubar,
-				       GtkWidget    *menu)
+panel_menu_bar_show_menu (PanelMenuBar *menubar,
+			  GtkWidget    *menu)
 {
 	gtk_menu_set_screen (GTK_MENU (menu),
 			     gtk_widget_get_screen (GTK_WIDGET (menubar)));
-
-	if (!menu_need_reread (menu))
-		return;
-
-	while (GTK_MENU_SHELL (menu)->children)
-		gtk_widget_destroy (GTK_MENU_SHELL (menu)->children->data);
-
-	create_root_menu (menu, menubar->priv->panel, TRUE, MENU_FLAGS, FALSE);
 }
 
 static void
-panel_menu_bar_append_action_item (PanelMenuBar          *menubar,
-				   PanelActionButtonType  action_type,
-				   GCallback              callback)
+panel_menu_bar_connect_menu_signals (PanelMenuBar  *menubar,
+				     GtkWidget    **menu)
 {
-	GtkWidget *item;
-
-	item = gtk_image_menu_item_new ();
-	setup_stock_menu_item (item,
-			       panel_menu_icon_get_size (),
-			       panel_action_get_stock_icon (action_type),
-			       panel_action_get_text (action_type),
-			       TRUE);
-
-	gtk_tooltips_set_tip (panel_tooltips,
-			      item,
-			      panel_action_get_tooltip (action_type),
-			      NULL);
-
-	gtk_menu_shell_append (GTK_MENU_SHELL (menubar->priv->actions_menu), item);
-	g_signal_connect (item, "activate", callback, NULL);
-	g_signal_connect (G_OBJECT (item), "button_press_event",
+	/* intercept all right button clicks makes sure they don't
+	   go to the object itself */
+	g_signal_connect (*menu, "button_press_event",
 			  G_CALLBACK (menu_dummy_button_press_event), NULL);
-	setup_internal_applet_drag (item,
-				    panel_action_get_drag_id (action_type));
+
+	g_signal_connect (*menu, "destroy", G_CALLBACK (gtk_widget_destroyed),
+			  menu);
+
+	g_signal_connect_swapped (*menu, "show",
+				  G_CALLBACK (panel_menu_bar_show_menu),
+				  menubar);
+
+	g_signal_connect_swapped (*menu, "hide",
+				  G_CALLBACK (gtk_menu_shell_deselect), 
+				  menubar);
 }
 
 static void
@@ -104,88 +88,76 @@ panel_menu_bar_recreate_actions_menu (PanelMenuBar *menubar)
 {
 	if (menubar->priv->actions_menu) {
 		gtk_widget_destroy (menubar->priv->actions_menu);
-		menubar->priv->actions_menu = NULL;
-
-		panel_menu_bar_append_actions_menu (menubar);
+		menubar->priv->actions_menu = panel_menu_bar_create_actions_menu (menubar);
+		panel_menu_bar_connect_menu_signals (menubar,
+						     &menubar->priv->actions_menu);
+		gtk_menu_item_set_submenu (GTK_MENU_ITEM (menubar->priv->actions_item),
+					   menubar->priv->actions_menu);
 	}
 }
 
-static void
-panel_menu_bar_append_actions_menu (PanelMenuBar *menubar)
+static GtkWidget *
+panel_menu_bar_create_actions_menu (PanelMenuBar *menubar)
 {
+	GtkWidget *actions_menu;
 	GtkWidget *item;
-	gboolean   enable_log_out;
-	gboolean   enable_lock_screen;
+	gboolean   separator_inserted;
+	gboolean   last_is_separator;
 
-	if (!menubar->priv->actions_menu) {
-		menubar->priv->actions_menu = panel_create_menu ();
+	actions_menu = panel_create_menu ();
 
-		/* intercept all right button clicks makes sure they don't
-	   	go to the object itself */
-		g_signal_connect (G_OBJECT (menubar->priv->actions_menu), "button_press_event",
-			  	G_CALLBACK (menu_dummy_button_press_event), NULL);
+	last_is_separator = FALSE;
 
-		g_signal_connect (menubar->priv->actions_menu, "destroy",
-			  	G_CALLBACK (gtk_widget_destroyed),
-			  	&menubar->priv->actions_menu);
+	item = menu_create_action_item (PANEL_ACTION_RUN);
+	if (item != NULL) {
+		gtk_menu_shell_append (GTK_MENU_SHELL (actions_menu), item);
+		add_menu_separator (actions_menu);
+		last_is_separator = TRUE;
 	}
 
-	if (!panel_lockdown_get_disable_command_line ()) {
-		panel_menu_bar_append_action_item (menubar,
-						   PANEL_ACTION_RUN,
-						   G_CALLBACK (panel_action_run_program));
-
-		item = gtk_separator_menu_item_new ();
-		gtk_menu_shell_append (GTK_MENU_SHELL (menubar->priv->actions_menu), item);
-		gtk_widget_show (item);
+	if (panel_is_program_in_path  ("gnome-search-tool")) {
+		item = menu_create_action_item (PANEL_ACTION_SEARCH);
+		if (item != NULL) {
+			gtk_menu_shell_append (GTK_MENU_SHELL (actions_menu),
+					       item);
+			last_is_separator = FALSE;
+		}
 	}
 
-	if (panel_is_program_in_path  ("gnome-search-tool"))
-		panel_menu_bar_append_action_item (menubar,
-						   PANEL_ACTION_SEARCH,
-						   G_CALLBACK (panel_action_search));
-
-	panel_recent_append_documents_menu (menubar->priv->actions_menu);
+	panel_recent_append_documents_menu (actions_menu);
 
 	if (panel_is_program_in_path ("gnome-screenshot")) {
-		item = gtk_separator_menu_item_new ();
-		gtk_menu_shell_append (GTK_MENU_SHELL (menubar->priv->actions_menu), item);
-		gtk_widget_show (item);
-
-		panel_menu_bar_append_action_item (menubar,
-						   PANEL_ACTION_SCREENSHOT,
-						   G_CALLBACK (panel_action_screenshot));
+		item = menu_create_action_item (PANEL_ACTION_SCREENSHOT);
+		if (item != NULL) {
+			if (!last_is_separator)
+				add_menu_separator (actions_menu);
+			gtk_menu_shell_append (GTK_MENU_SHELL (actions_menu),
+					       item);
+			last_is_separator = FALSE;
+		}
 	}
 
-	enable_log_out = !panel_lockdown_get_disable_log_out ();
-	enable_lock_screen =
-		(!panel_lockdown_get_disable_lock_screen () &&
-		 panel_is_program_in_path  ("xscreensaver"));
-		
+	separator_inserted = FALSE;
 
-	if (enable_log_out || enable_lock_screen) {
-		item = gtk_separator_menu_item_new ();
-		gtk_menu_shell_append (GTK_MENU_SHELL (menubar->priv->actions_menu), item);
-		gtk_widget_show (item);
+	item = menu_create_action_item (PANEL_ACTION_LOCK);
+	if (item != NULL) {
+		add_menu_separator (actions_menu);
+		gtk_menu_shell_append (GTK_MENU_SHELL (actions_menu), item);
+		separator_inserted = TRUE;
 	}
 
-	if (enable_lock_screen)
-		panel_menu_bar_append_action_item (menubar,
-						   PANEL_ACTION_LOCK,
-						   G_CALLBACK (panel_action_lock_screen));
+	if (panel_is_program_in_path ("xscreensaver")) {
+		item = menu_create_action_item (PANEL_ACTION_LOGOUT);
 
-	if (enable_log_out) {
-		panel_menu_bar_append_action_item (menubar,
-						   PANEL_ACTION_LOGOUT,
-						   G_CALLBACK (panel_action_logout));
+		if (item != NULL) {
+			if (!separator_inserted)
+				add_menu_separator (actions_menu);
+			gtk_menu_shell_append (GTK_MENU_SHELL (actions_menu),
+					       item);
+		}
 	}
 
-	if (!menubar->priv->actions_item) {
-		menubar->priv->actions_item = gtk_menu_item_new_with_label (_("Actions"));
-		gtk_menu_shell_append (GTK_MENU_SHELL (menubar), menubar->priv->actions_item);
-	}
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menubar->priv->actions_item),
-				   menubar->priv->actions_menu);
+	return actions_menu;
 }
 
 static void
@@ -199,35 +171,28 @@ panel_menu_bar_instance_init (PanelMenuBar      *menubar,
 
 	menubar->priv->info = NULL;
 
+	menubar->priv->applications_menu = create_applications_menu ("applications.menu", NULL);
+	panel_menu_bar_connect_menu_signals (menubar,
+					     &menubar->priv->applications_menu);
+
+	menubar->priv->actions_menu = panel_menu_bar_create_actions_menu (menubar);
+	panel_menu_bar_connect_menu_signals (menubar,
+					     &menubar->priv->actions_menu);
+
 	item = gtk_image_menu_item_new_with_label (_("Applications"));
-        image = gtk_image_new_from_stock (
-			PANEL_STOCK_GNOME_LOGO,
-			panel_menu_bar_icon_get_size ());
+	image = gtk_image_new_from_stock (PANEL_STOCK_GNOME_LOGO,
+					  panel_menu_bar_icon_get_size ());
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
 
-	menubar->priv->applications_menu =
-		create_root_menu (NULL, NULL, TRUE, MENU_FLAGS, FALSE);
-
-	/* intercept all right button clicks makes sure they don't
-	   go to the object itself */
-	g_signal_connect (G_OBJECT (menubar->priv->applications_menu), "button_press_event",
-			  G_CALLBACK (menu_dummy_button_press_event), NULL);
-
-	g_signal_connect (menubar->priv->applications_menu, "destroy",
-			  G_CALLBACK (gtk_widget_destroyed),
-			  &menubar->priv->applications_menu);
-
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item),
 				   menubar->priv->applications_menu);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menubar), item);
 
-	g_signal_connect_swapped (menubar->priv->applications_menu, "show",
-				  G_CALLBACK (panel_menu_bar_show_applications_menu), menubar);
-	g_signal_connect_swapped (menubar->priv->applications_menu, "hide",
-				  G_CALLBACK (gtk_menu_shell_deselect), menubar);
-	menubar->priv->actions_menu = NULL;
-	menubar->priv->actions_item = NULL;
-	panel_menu_bar_append_actions_menu (menubar);
+	menubar->priv->actions_item = gtk_menu_item_new_with_label (_("Actions"));
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menubar->priv->actions_item),
+				   menubar->priv->actions_menu);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menubar),
+			       menubar->priv->actions_item);
 
 	panel_lockdown_notify_add (G_CALLBACK (panel_menu_bar_recreate_actions_menu),
 				   menubar);
@@ -251,14 +216,37 @@ panel_menu_bar_parent_set (GtkWidget *widget,
 				       menubar->priv->panel);
 }
 
+static void
+panel_menu_bar_size_allocate (GtkWidget     *widget,
+			      GtkAllocation *allocation)
+{
+	GtkAllocation old_allocation;
+
+	old_allocation.x      = widget->allocation.x;
+	old_allocation.y      = widget->allocation.y;
+	old_allocation.width  = widget->allocation.width;
+	old_allocation.height = widget->allocation.height;
+
+	GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, allocation);
+
+	if (old_allocation.x      == allocation->x &&
+	    old_allocation.y      == allocation->y &&
+	    old_allocation.width  == allocation->width &&
+	    old_allocation.height == allocation->height)
+		return;
+
+	panel_menu_bar_change_background (PANEL_MENU_BAR (widget));
+}
+
 static void
 panel_menu_bar_class_init (PanelMenuBarClass *klass)
 {
-	GtkWidgetClass *widget_class  = (GtkWidgetClass *) klass;
+	GtkWidgetClass *widget_class = (GtkWidgetClass *) klass;
 
 	parent_class = g_type_class_peek_parent (klass);
 
 	widget_class->parent_set = panel_menu_bar_parent_set;
+	widget_class->size_allocate = panel_menu_bar_size_allocate;
 
 	g_type_class_add_private (klass, sizeof (PanelMenuBarPrivate));
 
@@ -393,3 +381,67 @@ panel_menu_bar_popup_menu (PanelMenuBar *menubar,
 	gtk_menu_shell_select_item (menu_shell,
 				    gtk_menu_get_attach_widget (menu));
 }
+
+void panel_menu_bar_change_background (PanelMenuBar *menubar)
+{
+	GtkRcStyle       *rc_style;
+	GtkStyle         *style;
+	const PanelColor *color;
+	GdkGC            *gc;
+	GdkPixmap        *pixmap;
+	const GdkPixmap  *bg_pixmap;
+
+	/* reset style */
+	gtk_widget_set_style (GTK_WIDGET (menubar), NULL);
+	rc_style = gtk_rc_style_new ();
+	gtk_widget_modify_style (GTK_WIDGET (menubar), rc_style);
+	g_object_unref (rc_style);
+
+	switch (panel_background_get_type (&menubar->priv->panel->background)) {
+	case PANEL_BACK_NONE:
+		break;
+	case PANEL_BACK_COLOR:
+		color = panel_background_get_color (&menubar->priv->panel->background);
+		if (color->alpha == 0xffff) {
+			gtk_widget_modify_bg (GTK_WIDGET (menubar),
+					      GTK_STATE_NORMAL, &(color->gdk));
+			break;
+		}
+		/* else, we have an image, so don't break */
+	case PANEL_BACK_IMAGE:
+		bg_pixmap = panel_background_get_pixmap (&menubar->priv->panel->background);
+		if (!bg_pixmap)
+			return;
+
+		gc = gdk_gc_new (GTK_WIDGET (menubar)->window);
+		g_return_if_fail (GDK_IS_GC (gc));
+
+		pixmap = gdk_pixmap_new (GTK_WIDGET (menubar)->window,
+					 GTK_WIDGET (menubar)->allocation.width,
+					 GTK_WIDGET (menubar)->allocation.height,
+					 -1);
+
+		gdk_draw_drawable (GDK_DRAWABLE (pixmap),
+				   gc, 
+				   GDK_DRAWABLE (bg_pixmap),
+				   GTK_WIDGET (menubar)->allocation.x,
+				   GTK_WIDGET (menubar)->allocation.y,
+				   0, 0,
+				   GTK_WIDGET (menubar)->allocation.width,
+				   GTK_WIDGET (menubar)->allocation.height);
+
+		g_object_unref (gc);
+
+		style = gtk_style_copy (GTK_WIDGET (menubar)->style);
+		if (style->bg_pixmap[GTK_STATE_NORMAL])
+			g_object_unref (style->bg_pixmap[GTK_STATE_NORMAL]);
+		style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref (pixmap);
+		gtk_widget_set_style (GTK_WIDGET (menubar), style);
+
+		g_object_unref (pixmap);
+		break;
+	default:
+		g_assert_not_reached ();
+		break;
+	}
+}
