@@ -17,25 +17,20 @@
 
 #include <gdk/gdkkeysyms.h>
 
-#include <libgnome/libgnome.h>
 #include <libbonobo.h>
+#include <libgnomeui/gnome-icon-lookup.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
-#include <libgnomevfs/gnome-vfs-mime.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
 #include <libgnomevfs/gnome-vfs-file-info.h>
-#include <libgnomeui/gnome-window-icon.h>
 
 #include "panel.h"
 
 #include "applet.h"
 #include "drawer.h"
 #include "button-widget.h"
-#include "distribution.h"
 #include "launcher.h"
-#include "menu-fentry.h"
-#include "menu-util.h"
-#include "menu.h"
+#include "panel-context-menu.h"
 #include "panel-util.h"
 #include "panel-config-global.h"
 #include "panel-gconf.h"
@@ -153,9 +148,17 @@ void
 back_change (AppletInfo  *info,
 	     PanelWidget *panel)
 {
-	if (info->type == PANEL_OBJECT_BONOBO)
+	switch (info->type) {
+	case PANEL_OBJECT_BONOBO:
 		panel_applet_frame_change_background (
 			PANEL_APPLET_FRAME (info->widget), panel->background.type);
+		break;
+	case PANEL_OBJECT_MENU_BAR:
+		panel_menu_bar_change_background (PANEL_MENU_BAR (info->widget));
+		break;
+	default:
+		break;
+	}
 }
 
 static void
@@ -225,7 +228,8 @@ deactivate_idle (gpointer data)
 }
 
 static void
-menu_deactivate(GtkWidget *w, PanelData *pd)
+context_menu_deactivate (GtkWidget *w,
+			 PanelData *pd)
 {
 	if (pd->deactivate_idle == 0)
 		pd->deactivate_idle = g_idle_add (deactivate_idle, pd);
@@ -233,6 +237,13 @@ menu_deactivate(GtkWidget *w, PanelData *pd)
 	panel_toplevel_pop_autohide_disabler (PANEL_TOPLEVEL (pd->panel));
 }
 
+static void
+context_menu_show (GtkWidget *w,
+		   PanelData *pd)
+{
+	panel_toplevel_push_autohide_disabler (PANEL_TOPLEVEL (pd->panel));
+}
+
 static void
 panel_recreate_context_menu (PanelData *pd)
 {
@@ -284,10 +295,15 @@ static GtkWidget *
 panel_menu_get (PanelWidget *panel, PanelData *pd)
 {
 	if (!pd->menu) {
-		pd->menu = g_object_ref (create_panel_context_menu (panel));
-		gtk_object_sink (GTK_OBJECT (pd->menu));
-		g_signal_connect (pd->menu, "deactivate",
-				  G_CALLBACK (menu_deactivate), pd);
+		pd->menu = g_object_ref (panel_context_menu_create (panel));
+		if (pd->menu) {
+			gtk_object_sink (GTK_OBJECT (pd->menu));
+			g_signal_connect (pd->menu, "deactivate",
+					  G_CALLBACK (context_menu_deactivate),
+					  pd);
+			g_signal_connect (pd->menu, "show",
+					  G_CALLBACK (context_menu_show), pd);
+		}
 	}
 
 	return pd->menu;
@@ -338,14 +354,13 @@ panel_popup_menu (PanelToplevel *toplevel,
 		panel_data->insertion_pos = -1;
 	
 	menu = make_popup_panel_menu (panel_widget);
-
-	panel_toplevel_push_autohide_disabler (toplevel);
+	if (!menu)
+		return FALSE;
 
 	gtk_menu_set_screen (GTK_MENU (menu),
 			     gtk_window_get_screen (GTK_WINDOW (toplevel)));
 
-	gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
-			panel_menu_position, toplevel, button, activate_time);
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, button, activate_time);
 
 	return TRUE;
 }
@@ -481,96 +496,58 @@ drop_menu (PanelWidget *panel,
 	if (!panel_profile_id_lists_are_writable ())
 		return FALSE;
 
-	panel_menu_button_create (panel->toplevel, position, menu_path,
-				  menu_path != NULL, menu_path);
+	panel_menu_button_create (panel->toplevel,
+				  position,
+				  menu_path,
+				  menu_path != NULL,
+				  NULL);
 
 	return TRUE;
 }
 
 static gboolean
-drop_nautilus_uri (PanelWidget *panel,
-		   int          position,
-		   const char  *uri,
-		   const char  *icon)
+drop_uri (PanelWidget *panel,
+	  int          position,
+	  const char  *uri,
+	  const char  *icon)
 {
-	char *quoted;
-	char *exec;
-	char *base;
-	char **split;
+	char  *base;
 
 	if (!panel_profile_id_lists_are_writable ())
 		return FALSE;
 
-	quoted = g_shell_quote (uri);
-
-	/* Escape any "%" as "%%" */
-	split = g_strsplit (quoted, "%", -1);
-	g_free (quoted);
-	quoted = g_strjoinv ("%%", split);
-	g_strfreev (split);
-
-	/* Add -- to avoid the possibility of filenames which would
-	 * be interpreted as command line arguments
-	 */
-	exec = g_strdup_printf ("nautilus -- %s", quoted);
-	g_free (quoted);
-
 	base = g_path_get_basename (uri);
 
 	panel_launcher_create_from_info (
-		panel->toplevel, position, TRUE, exec, base, uri, icon);
+		panel->toplevel, position, FALSE, uri, base, uri, icon);
 
-	g_free (exec);
 	g_free (base);
 
 	return TRUE;
 }
 
 static gboolean
-drop_directory (PanelWidget *panel, int pos, const char *dir)
+drop_urilist (PanelWidget *panel,
+	      int          pos,
+	      char        *urilist)
 {
-	char *tmp;
+	char     **uris;
+	gboolean   success;
+	int        i;
 
-	/* not filename, but path, these are uris, not local
-	 * files */
-	tmp = g_build_path ("/", dir, ".directory", NULL);
-	if (panel_uri_exists (tmp)) {
-		g_free (tmp);
-		return drop_menu (panel, pos, dir);
-	}
-	g_free (tmp);
+	uris = g_uri_list_extract_uris (urilist);
 
-	tmp = g_build_path ("/", dir, ".order", NULL);
-	if (panel_uri_exists (tmp)) {
-		g_free (tmp);
-		return drop_menu (panel, pos, dir);
-	}
-	g_free (tmp);
-
-	if (panel_is_program_in_path ("nautilus")) {
-		/* nautilus */
-		return drop_nautilus_uri (panel, pos, dir, "gnome-fs-directory.png");
-	} else {
-		return drop_menu (panel, pos, dir);
-	}
-}
-
-static gboolean
-drop_urilist (PanelWidget *panel, int pos, char *urilist)
-{
-	gboolean success = TRUE;
-	GList *li, *files;
-
-	files = gnome_vfs_uri_list_parse (urilist);
-
-	for (li = files; li; li = li->next) {
-		GnomeVFSURI *vfs_uri = li->data;
-		gchar *uri = gnome_vfs_uri_to_string (vfs_uri, GNOME_VFS_URI_HIDE_NONE);
-		const char *mimetype;
-		char *basename;
-		char *dirname;
-		char *filename;
+	success = TRUE;
+	for (i = 0; uris[i]; i++) {
+		GnomeVFSURI      *vfs_uri;
 		GnomeVFSFileInfo *info;
+		GnomeVFSResult    res;
+		const char       *uri;
+		char             *basename;
+		char             *dirname;
+		char             *filename;
+
+		uri = uris[i];
 
 		if (strncmp (uri, "http:", strlen ("http:")) == 0 ||
 		    strncmp (uri, "https:", strlen ("https:")) == 0 ||
@@ -587,77 +564,75 @@ drop_urilist (PanelWidget *panel, int pos, char *urilist)
 			continue;
 		}
 
-		mimetype = gnome_vfs_mime_type_from_name (uri);
+		if (!(vfs_uri = gnome_vfs_uri_new (uri))) {
+			success = FALSE;
+			continue;
+		}
+
 		basename = gnome_vfs_uri_extract_short_path_name (vfs_uri);
 		dirname = gnome_vfs_uri_extract_dirname (vfs_uri);
+
 		info = gnome_vfs_file_info_new ();
+		res = gnome_vfs_get_file_info_uri (vfs_uri, info,
+						   GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
+						   GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
+		if (res == GNOME_VFS_OK) {
+			if (info->mime_type &&
+			    !strncmp (info->mime_type, "image", sizeof ("image") - 1)) {
+				if (!set_background_image_from_uri (panel->toplevel, uri))
+					success = FALSE;
+			} else if (info->mime_type != NULL &&
+				   (!strcmp (info->mime_type, "application/x-gnome-app-info") ||
+				    !strcmp (info->mime_type, "application/x-desktop") ||
+				    !strcmp (info->mime_type, "application/x-kde-app-info"))) {
+				if (panel_profile_id_lists_are_writable ())
+					panel_launcher_create (panel->toplevel, pos, uri);
+				else
+					success = FALSE;
+			} else if (info->type != GNOME_VFS_FILE_TYPE_DIRECTORY &&
+				   info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_PERMISSIONS &&
+				   info->permissions & (GNOME_VFS_PERM_USER_EXEC |
+							GNOME_VFS_PERM_GROUP_EXEC |
+							GNOME_VFS_PERM_OTHER_EXEC) &&
+				   (filename = g_filename_from_uri (uri, NULL, NULL))) {
+				if (panel_profile_id_lists_are_writable ())
+					/* executable and local, so add a launcher with it
+					 */
+					ask_about_launcher (filename, panel, pos, TRUE);
+				else
+					success = FALSE;
+				g_free (filename);
+			} else {
+				char *icon;
 
-		if (gnome_vfs_get_file_info_uri (vfs_uri, info,
-						 GNOME_VFS_FILE_INFO_DEFAULT) != GNOME_VFS_OK) {
-			gnome_vfs_file_info_unref (info);
-			info = NULL;
-		}
+				icon = gnome_icon_lookup (panel_icon_theme,
+							  NULL, NULL, NULL,
+							  info, info->mime_type,
+							  GNOME_ICON_LOOKUP_FLAGS_NONE,
+							  NULL);
 
-		if (mimetype &&
-		    !strncmp (mimetype, "image", sizeof ("image") - 1)) {
-			if ( ! set_background_image_from_uri (panel->toplevel, uri))
-				success = FALSE;
-		} else if (basename != NULL &&
-			   strcmp (basename, ".directory") == 0 &&
-			   dirname != NULL) {
-			/* This is definately a menu */
-			char *menu_uri = g_strconcat (vfs_uri->method_string, ":",
-						      dirname, NULL);
-			if ( ! drop_menu (panel, pos, menu_uri))
-				success = FALSE;
-			g_free (menu_uri);
-		} else if (mimetype != NULL &&
-			   (strcmp(mimetype, "application/x-gnome-app-info") == 0 ||
-			    strcmp(mimetype, "application/x-desktop") == 0 ||
-			    strcmp(mimetype, "application/x-kde-app-info") == 0)) {
-			if (panel_profile_id_lists_are_writable ())
-				panel_launcher_create (panel->toplevel, pos, uri);
-			else
-				success = FALSE;
-		} else if (info != NULL &&
-			   info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) {
-			if ( ! drop_directory (panel, pos, uri))
-				success = FALSE;
-		} else if (info != NULL &&
-			   info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_PERMISSIONS &&
-			   info->permissions &
-			     (GNOME_VFS_PERM_USER_EXEC |
-			      GNOME_VFS_PERM_GROUP_EXEC |
-			      GNOME_VFS_PERM_OTHER_EXEC) &&
-			   (filename = g_filename_from_uri (uri, NULL, NULL)) != NULL) {
-			if (panel_profile_id_lists_are_writable ())
-				/* executable and local, so add a launcher with
-				 * it */
-				ask_about_launcher (filename, panel, pos, TRUE);
-			else
-				success = FALSE;
-			g_free (filename);
+				if (!icon)
+					icon = g_strdup ("gnome-unknown.png");
+
+				if (!drop_uri (panel, pos, uri, icon))
+					success = FALSE;
+
+				g_free (icon);
+			}
 		} else {
-			/* FIXME: add a launcher that will launch the app
-			 * associated with this file */
-			/* FIXME: For now just add a launcher that launches
-			 * nautilus on this uri */
-			const char *icon = NULL;
-			if (mimetype != NULL)
-		        	icon = gnome_vfs_mime_get_icon (mimetype);
-			if (icon == NULL)
-				icon = "gnome-unknown.png";
-			if ( ! drop_nautilus_uri (panel, pos, uri, icon))
+			if (!drop_uri (panel, pos, uri, "gnome-unknown.png"))
 				success = FALSE;
 		}
-		if (info != NULL)
-			gnome_vfs_file_info_unref (info);
+
+		gnome_vfs_file_info_unref (info);
+
 		g_free (basename);
 		g_free (dirname);
-		g_free (uri);
+
+		gnome_vfs_uri_unref (vfs_uri);
 	}
 
-	gnome_vfs_uri_list_free (files);
+	g_strfreev (uris);
 
 	return success;
 }
@@ -961,7 +936,6 @@ drag_motion_cb (GtkWidget	   *widget,
 	do_highlight (widget, TRUE);
 
 	panel_toplevel_unhide (toplevel);
-	panel_toplevel_queue_auto_hide (toplevel);
 
 	return TRUE;
 }
@@ -990,7 +964,12 @@ drag_leave_cb (GtkWidget	*widget,
 	       guint             time,
 	       Launcher         *launcher)
 {
+	PanelToplevel *toplevel;
+
 	do_highlight (widget, FALSE);
+
+	toplevel = PANEL_TOPLEVEL (widget);
+	panel_toplevel_queue_auto_hide (toplevel);
 }
 
 void
@@ -1030,7 +1009,7 @@ panel_receive_dnd_data (PanelWidget      *panel,
 		}
 		break;
 	case TARGET_DIRECTORY:
-		success = drop_directory (panel, pos, (char *)selection_data->data);
+		success = drop_uri (panel, pos, (char *)selection_data->data, "gnome-fs-directory.png");
 		break;
 	case TARGET_APPLET:
 		if (!selection_data->data) {
@@ -1176,19 +1155,6 @@ panel_setup (PanelToplevel *toplevel)
 	return pd;
 }
 
-void
-panel_register_window_icon (void)
-{
-	char *panel_icon;
-
-	panel_icon = panel_pixmap_discovery ("gnome-panel.png", FALSE);
-
-	if (panel_icon) {
-		gnome_window_icon_set_default_from_file (panel_icon);
-		g_free (panel_icon);
-	}
-}
-
 GdkScreen *
 panel_screen_from_panel_widget (PanelWidget *panel)
 {
@@ -1263,22 +1229,38 @@ panel_deletion_dialog (PanelToplevel *toplevel)
 {
 
 	GtkWidget *dialog;
+	char *text1;
+	char *text2;
+	char *title;
+
+	if (panel_toplevel_get_is_attached (toplevel)) {
+		text1 = _("Delete this drawer?");
+		text2 = _("When a drawer is deleted, the drawer and its\n"
+			 "settings are lost.");
+		title = _("Delete Drawer");
+	} else {
+		text1 = _("Delete this panel?");
+		text2 = _("When a panel is deleted, the panel and its\n"
+			 "settings are lost.");
+		title = _("Delete Panel");
+	}
 
 	dialog = gtk_message_dialog_new (
 			GTK_WINDOW (toplevel),
 			GTK_DIALOG_MODAL,
 			GTK_MESSAGE_QUESTION,
 			GTK_BUTTONS_NONE,
-			_("When a panel is deleted, the panel "
-			"and its\n settings are lost. "
-			"Delete this panel?"));
+			text1);
+	
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+	                                          text2);	
 	gtk_dialog_add_buttons (GTK_DIALOG (dialog),
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 				GTK_STOCK_DELETE, GTK_RESPONSE_OK,
 				NULL);
 
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
-	gtk_window_set_title (GTK_WINDOW (dialog), _("Delete Panel"));
+	gtk_window_set_title (GTK_WINDOW (dialog), title);
 
 	gtk_window_set_wmclass (GTK_WINDOW (dialog),
 				"panel_remove_query", "Panel");
@@ -1293,6 +1275,7 @@ panel_deletion_dialog (PanelToplevel *toplevel)
                            toplevel);
 
 	g_object_set_data (G_OBJECT (toplevel), "panel-delete-dialog", dialog);
+	panel_toplevel_push_autohide_disabler (toplevel);
 
 	return dialog;
 }
